/*
 * $Revision: 1.1 $
 */

// 320x240 pixel 256-color LCD functions 

#include "s3c44b0x.h"
#include "lcdlib.h"

#define M5D(n) ((n) & 0x1fffff)

#define ARRAY_SIZE_COLOR (SCR_XSIZE/1*SCR_YSIZE)

#define HOZVAL (LCD_XSIZE/4-1) 
#define HOZVAL_COLOR (LCD_XSIZE*3/8-1) 
#define LINEVAL (LCD_YSIZE-1)  
#define MVAL 13 
#define MVAL_USED 0
#define CLKVAL_COLOR 4

unsigned char (*frameBuffer256)[SCR_XSIZE];


void Lcd_Init()
{
  //320x240 1bit/1pixel LCD
  rPDATC=rPDATC&~(1<<8)|(1<<8);	    //GPE5=H	 lcd enable
  rPCONC=rPCONC&~(0xff<<8)|(0xff<<8);	//GPC[4:7] => VD[7:4]    


  if (frameBuffer256 == 0)
  {
    frameBuffer256=(unsigned char (*)[SCR_XSIZE])0xc000000;
  }

  rLCDCON1=(0)|(2<<5)|(MVAL_USED<<7)|(0x3<<8)|(0x3<<10)|(CLKVAL_COLOR<<12);
  // disable,8B_SNGL_SCAN,WDLY=8clk,WLH=8clk,
  rLCDCON2=(LINEVAL)|(HOZVAL_COLOR<<10)|(10<<21);  
  // LINEBLANK=10 (without any calculation) 
  rLCDSADDR1 = (0x3<<27) |
               (((unsigned int)frameBuffer256>>22)<<21) |
               M5D((unsigned int)frameBuffer256>>1);
  // 256-color, LCDBANK, LCDBASEU
  rLCDSADDR2 = M5D((((unsigned int)frameBuffer256+(SCR_XSIZE*LCD_YSIZE))>>1)) | (MVAL<<21);
               rLCDSADDR3= (LCD_XSIZE/2) | ( ((SCR_XSIZE-LCD_XSIZE)/2)<<9 );

  //The following values has to be changed for better display.
  rREDLUT = 0xfdb96420;
  rGREENLUT = 0xfdb96420;
  rBLUELUT = 0xfb40;

  rDITHMODE = 0x0;
  rDP1_2 = 0xa5a5;      
  rDP4_7 = 0xba5da65;
  rDP3_5 = 0xa5a5f;
  rDP2_3 = 0xd6b;
  rDP5_7 = 0xeb7b5ed;
  rDP3_4 = 0x7dbe;
  rDP4_5 = 0x7ebdf;
  rDP6_7 = 0x7fdfbfe;

  // enable,8B_SNGL_SCAN,WDLY=8clk,WLH=8clk,
  rLCDCON1 = (1)|(2<<5)|(MVAL_USED<<7)|(0x3<<8)|(0x3<<10)|(CLKVAL_COLOR<<12);
}


#define PM_CLK_SET  (rPDATC = rPDATC | 0x200) 	// PM_CLK rPDATC9
#define PMDATA_SET  (rPDATC = rPDATC | 0x400) 	// PMDATA rPDATC10
#define PM_nRST_SET (rPDATC = rPDATC | 0x800) 	// PM_nRST rPDATC11
#define PM_CLK_CLR  (rPDATC = rPDATC & 0xfdff)
#define PMDATA_CLR  (rPDATC = rPDATC & 0xfbff)
#define PM_nRST_CLR (rPDATC = rPDATC & 0xf7ff)
#define PM_ALL_CLR  (rPDATC = rPDATC & 0xf1ff)


void PCLK_GEN(void)
{
  PM_CLK_SET;
  PM_CLK_CLR;
}


void SetContrast(unsigned char value)            // DS1267 SETTING
{
  unsigned int i,j,temp;

  temp = value*0x4;      //2bit left shift   if value =0xff, temp =0x03fc

  PM_nRST_SET;	       //Start condision
  PCLK_GEN();            //Stack selection bit (null)

  for(i=0;i<8;i++)       // ch1 of potentionmeter value 8bit (null) ,clock only
  {	
    PCLK_GEN();
  }

  for(i=0;i<8;i++)       // VEE contrast value set
  {	
    temp <<= 1;         // shift left 1 bit
    j = temp;

    PMDATA_CLR;

    j &= 0x400;
    rPDATC=rPDATC|j;

    PCLK_GEN();
  }

  PM_ALL_CLR;
}
      


