/*
 * $Revision: 1.2 $
 */

//
// Aiji SCORPIO example program.
// Displays a moving pattern on the LED's connected
// to GPIO0, GPIO2, GPIO3 and GPIO4.
//

#include <stdio.h>
#include <inarm.h>
#include "reg.h"

void mmu_setup();  // Assembler function to setup MMU.

int tick; // Delay tick, decremented every 1 ms by timer 0 interrupt.

// Moving LED pattern.
int led_table[] = { 1, 2, 4, 8, 4, 2 };

//
// IRQ interrupt handler.
//
__irq __arm void irq_handler()
{
  int vec;

  vec = rIVEC_ADDR; // Get vector offset.
  vec >>= 2; // To get vecor number.

  // Check if timer 0 interrupt.
  if (vec == 2)
  {
    // Timer0 interrupt will be called 1000 Hz rate.
    // Stop at zero.
    if (tick)
      --tick;
    rTCON0 &= ~0x10; // Clear pending interrupt at timer 0.
  }

  // Clear interrupt controller service pending bit.
  rIRQISPC = 1 << vec;
}

//
// FIQ interrupt handler.
//
__fiq __arm void fiq_handler()
{
  int vec = rFVEC_ADDR;
  vec >>= 2; // To get vecor number.

  // Clear interrupt controller service pending bit.
  rFIQISPC = 1 << vec;
}

//
// Delay the specified number of milliseconds.
// Timebase is timer 0.
//
void delay_ms(int millisec)
{
  tick = millisec;
  while (tick) ;
}

//
// Displays a moving pattern on the LED's connected
// to GPIO0, GPIO2, GPIO3 and GPIO4.
//
main()
{
  // Setup the memory management unit.
  mmu_setup();

  // Setup the interrupt controller.
  rINTCON = 0x0f; // Global mask, semi-vectored, IRQ disable, FIQ disable.
  rINTMOD = 0; //IRQ mode.
  rINTLEVEL = 0xffffffff; // Level triggered  mode.
  rPOLARITY = 0; // Active high.
  rINTMSK = ~0x04; // Enable timer 0 interrupt.
  rINTCON = 0x0d; // Global mask, semi-vectored, IRQ enable, FIQ disable.

  // Enable IRQ and FIQ at CPU core.
  __enable_interrupt();

  // Setup timer 0.
  rTPSR0 = 40; // System clock is 40 MHz, get down to 1 MHz.
  rTDR0 = 1000; // Interrupt every millisecond.
  rTCON0 = 0x109; // Interrupt enabled, level triggered, start timer.

  for (;;)
  {
    int i;

    for (i = 0; i < 6; i++)
    {
      // LED's are connected to GPIO0, GPIO2, GPIO3 and GPIO4.
      rGIOPDATA |= 0x000F; // Clear LED's.
      rGIOPDATA &= ~led_table[i]; // Set LED pattern.
      delay_ms(200); // Show LED pattern for 200 ms.
    }
  }
}


