/*
 * $Revision: 1.1 $
 */

/**********************************************************
 * MODULE : reg.h 
 * PURPOSE : Define the registers of each module
 * VERSION : Version 0.30
 * 2/23/2000
 * 3/28/2000
 * 4/06/2000
 * 4/07/2000
 * 5/18/2000 - miscellaneous
 * 7/07/2000 - addition KMI1
 *    before alteration
 *    KMI : 0x0C083000 ~ 0x0C0833FF
 *    MISC: 0x0C084800 ~ 0x0C084BFF
 *    after alteration
 *    KMI0: 0x0C083000 ~ 0x0C0833FF
 *    KMI1: 0x0C084800 ~ 0x0C084BFF
 *    MISC: 0x0C084C00 ~ 0x0C084FFF
 *********************************************************/

#define rSCRBase     0x40000000     /* base of the System Configuration register */
#define rASICBase    0x0C000000	    /* base of all I/O module register */
#define rPTEAKBase   0x08F00000     /* preload TeakLite base address */

/* define the base of each I/O devices */
#define rARM920T     rASICBase
#define rMEMBase     (rASICBase+0x10000) /* Memory controller */
#define rDMABase     (rASICBase+0x20000) /* DMA controller */
#define rLCDBase     (rASICBase+0x30000) /* LCD controller */
#define rEMACBase    (rASICBase+0x40000) /* Ethernet MAC controller */
#define rXMEMBase    (rASICBase+0x50000) /* TeakLite X memory(16KB) */
#define rYMEMBase    (rASICBase+0x60000) /* TeakLite Y memory(8KB) */
#define rPMEMBase    (rASICBase+0x70000) /* TeakLite P memory(32KB) */
#define rAPBBase     (rASICBase+0x80000) /* AHB2APB bridge */

/* Preload TeakLite address */
#define rPPDataArea  (rPTEAKBase+0x00)   /* preload TeakLite P data area */
#define rPXDataArea  (rPTEAKBase+0x40000)/* preload TeakLite X data area */
#define rPYDataArea  (rPTEAKBase+0x60000)/* preload TeakLite Y data area */

/************ AHB2APB bridge I/O device register *************/ 
#define rBRIDGEBase  (rAPBBase+0x000)    /* APB Bridge interface */
#define rSCIBase     (rAPBBase+0x400)    /* Smart card interface */
#define rUSBBase     (rAPBBase+0x800)    /* USB */
#define rPPIBase     (rAPBBase+0xC00)    /* IEE1284, Parallel port */
#define rIICBase     (rAPBBase+0x1000)   /* IIC */
#define rTIMEBase    (rAPBBase+0x1400)   /* Timer */
#define rRTCBase     (rAPBBase+0x1800)   /* Real time clock */
#define rWDTBase     (rAPBBase+0x1C00)   /* Watch Dog Timer */
#define rIOPBase     (rAPBBase+0x2000)   /* Programmable I/O port */
#define rRPCBase     (rAPBBase+0x2400)   /* Memory remap */ 
#define rINTBase     (rAPBBase+0x2800)   /* Interrupt controller */
#define rSSPBase     (rAPBBase+0x2C00)   /* SSP interface */
#define rKMI0Base    (rAPBBase+0x3000)   /* KMI0 */
#define rUART0Base   (rAPBBase+0x3400)   /* UART0 */
#define rUART1Base   (rAPBBase+0x3800)   /* UART1 */
#define rPMBase      (rAPBBase+0x3C00)   /* Power manager */
#define rTEAKBase    (rAPBBase+0x4000)   /* Teaklite Z space */
#define rKMI1Base    (rAPBBase+0x4800)   /* KMI1 */
#define rMISCBase    (rAPBBase+0x4C00)   /* Miscellaneous */
#define rEXT0Base    (rAPBBase+0x5000)   /* External 0 */
#define rEXT1Base    (rAPBBase+0x5400)   /* External 1 */
#define rEXT2Base    (rAPBBase+0x5800)   /* External 2 */
#define rEXT3Base    (rAPBBase+0x5C00)   /* External 3 */
#define rPCMCIABase  (rAPBBase+0x9000)   /* Programmable I/O port */

/* Memory Controller(32bit)*/
#define rSDRAMCFG0    (*(volatile unsigned *)(rMEMBase+0x00))	/* SDRAM config0 register */
#define rSDRAMCFG1    (*(volatile unsigned *)(rMEMBase+0x04))	/* SDRAM config1 register */
#define rSDRAMRefresh (*(volatile unsigned *)(rMEMBase+0x08))	/* SDRAM refresh register */
#define rSDRAMWB      (*(volatile unsigned *)(rMEMBase+0x0C))	/* SDRAM WB timeout register */
// old static memory controller version
#define rSRAMCFG      (*(volatile unsigned *)(rMEMBase+0x400))
#define rROMCFG       (*(volatile unsigned *)(rMEMBase+0x404))
#define rFLASHCFG     (*(volatile unsigned *)(rMEMBase+0x408))
#define rBANKCFG      (*(volatile unsigned *)(rMEMBase+0x40C))
// new static memory controller version
#define rSMCBANK0     (*(volatile unsigned *)(rMEMBase+0x10))
#define rSMCBANK1     (*(volatile unsigned *)(rMEMBase+0x14))
#define rSMCBANK2     (*(volatile unsigned *)(rMEMBase+0x18))
#define rSMCBANK3     (*(volatile unsigned *)(rMEMBase+0x1C))

/* DMA Controller(32bit)*/ 
// DMA control register 
#define rDMACON0     (*(volatile unsigned *)(rDMABase+0x00))	
#define rDMACON1     (*(volatile unsigned *)(rDMABase+0x40))	
#define rDMACON2     (*(volatile unsigned *)(rDMABase+0x80))	
#define rDMACON3     (*(volatile unsigned *)(rDMABase+0xC0))	
#define rDMACON4     (*(volatile unsigned *)(rDMABase+0x100))	
#define rDMACON5     (*(volatile unsigned *)(rDMABase+0x140))	

// DMA Source Start address register 
#define rDMASAR0     (*(volatile unsigned *)(rDMABase+0x04))	
#define rDMASAR1     (*(volatile unsigned *)(rDMABase+0x44))	
#define rDMASAR2     (*(volatile unsigned *)(rDMABase+0x84))	
#define rDMASAR3     (*(volatile unsigned *)(rDMABase+0xC4))	
#define rDMASAR4     (*(volatile unsigned *)(rDMABase+0x104))	
#define rDMASAR5     (*(volatile unsigned *)(rDMABase+0x144))	

// DMA Destination address register
#define rDMADAR0     (*(volatile unsigned *)(rDMABase+0x08))
#define rDMADAR1     (*(volatile unsigned *)(rDMABase+0x48))
#define rDMADAR2     (*(volatile unsigned *)(rDMABase+0x88))
#define rDMADAR3     (*(volatile unsigned *)(rDMABase+0xC8))
#define rDMADAR4     (*(volatile unsigned *)(rDMABase+0x108))
#define rDMADAR5     (*(volatile unsigned *)(rDMABase+0x148))

// DMA Terminal Counter register 
#define rDMATCR0     (*(volatile unsigned *)(rDMABase+0x0C))
#define rDMATCR1     (*(volatile unsigned *)(rDMABase+0x4C))
#define rDMATCR2     (*(volatile unsigned *)(rDMABase+0x8C))
#define rDMATCR3     (*(volatile unsigned *)(rDMABase+0xCC))
#define rDMATCR4     (*(volatile unsigned *)(rDMABase+0x10C))
#define rDMATCR5     (*(volatile unsigned *)(rDMABase+0x14C))

// DMA pending & priority register
#define rDMAPRI      (*(volatile unsigned *)(rDMABase+0x1F8))
#define rDMAPND      (*(volatile unsigned *)(rDMABase+0x1FC))

/* LCD(primecell) Controller(32bit)*/
#define rLCDTIMMING0 (*(volatile unsigned *)(rLCDBase+0x00))
#define rLCDTIMMING1 (*(volatile unsigned *)(rLCDBase+0x04))
#define rLCDTIMMING2 (*(volatile unsigned *)(rLCDBase+0x08))
#define rLCDTIMMING3 (*(volatile unsigned *)(rLCDBase+0x0C))
#define rLCDUPBASE   (*(volatile unsigned *)(rLCDBase+0x10))
#define rLCDLPBASE   (*(volatile unsigned *)(rLCDBase+0x14))
#define rLCDINTREN   (*(volatile unsigned *)(rLCDBase+0x18))
#define rLCDCONTROL  (*(volatile unsigned *)(rLCDBase+0x1C))
#define rLCDSTATUS   (*(volatile unsigned *)(rLCDBase+0x20))
#define rLCDINTR     (*(volatile unsigned *)(rLCDBase+0x24))
#define rLCDUPCUCR   (*(volatile unsigned *)(rLCDBase+0x28))
#define rLCDLPCUCR   (*(volatile unsigned *)(rLCDBase+0x2C))
#define rLCDPALBase  (rLCDBase+0x200)

/* APB Base register map */ 
/* 1.APB pulse width control register */
#define rAPBCON0     (*(volatile unsigned *)(rBRIDGEBase+0x00))
#define rAPBCON1     (*(volatile unsigned *)(rBRIDGEBase+0x04))
#define rAPBCON2     (*(volatile unsigned *)(rBRIDGEBase+0x08))
#define rAPBCON3     (*(volatile unsigned *)(rBRIDGEBase+0x0C))

/* 2.Smart card interface */
#define rSCIDATA     (*(volatile unsigned *)(rSCIBase+0x00)) 
#define rSCICR0      (*(volatile unsigned *)(rSCIBase+0x04)) 
#define rSCICR1      (*(volatile unsigned *)(rSCIBase+0x08)) 
#define rSCICR2      (*(volatile unsigned *)(rSCIBase+0x0C)) 
#define rSCIIER      (*(volatile unsigned *)(rSCIBase+0x10)) 
#define rSCIRETRY    (*(volatile unsigned *)(rSCIBase+0x14)) 
#define rSCITIDE     (*(volatile unsigned *)(rSCIBase+0x18)) 
#define rSCITXCOUNT  (*(volatile unsigned *)(rSCIBase+0x1C)) 
#define rSCIRXCOUNT  (*(volatile unsigned *)(rSCIBase+0x20)) 
#define rSCIFR       (*(volatile unsigned *)(rSCIBase+0x24))
#define rSCIRXTIME   (*(volatile unsigned *)(rSCIBase+0x28)) 
#define rSCISTAT     (*(volatile unsigned *)(rSCIBase+0x2C))
#define rSCISTABLE   (*(volatile unsigned *)(rSCIBase+0x30)) 
#define rSCIATIME    (*(volatile unsigned *)(rSCIBase+0x34)) 
#define rSCIDTIME    (*(volatile unsigned *)(rSCIBase+0x38)) 
#define rSCIATRSTIME (*(volatile unsigned *)(rSCIBase+0x3C))
#define rSCIATRDTIME (*(volatile unsigned *)(rSCIBase+0x40))
#define rSCIBLKTIME  (*(volatile unsigned *)(rSCIBase+0x44))
#define rSCICHTIME   (*(volatile unsigned *)(rSCIBase+0x48))
#define rSCICLKICC   (*(volatile unsigned *)(rSCIBase+0x4C))
#define rSCIBAUD     (*(volatile unsigned *)(rSCIBase+0x50))
#define rSCIVALUE    (*(volatile unsigned *)(rSCIBase+0x54))
#define rSCICHGUARD  (*(volatile unsigned *)(rSCIBase+0x58))
#define rSCIBLKGUARD (*(volatile unsigned *)(rSCIBase+0x5C))
#define rSCISYNCCR   (*(volatile unsigned *)(rSCIBase+0x60))
#define rSCISYNCDATA (*(volatile unsigned *)(rSCIBase+0x64))
#define rSCIRAWSTAT  (*(volatile unsigned *)(rSCIBase+0x68))
#define rSCIIIR      (*(volatile unsigned *)(rSCIBase+0x6C))
#define rSCITCR      (*(volatile unsigned *)(rSCIBase+0x80))
#define rSCITMR      (*(volatile unsigned *)(rSCIBase+0x84))
#define rSCITISR     (*(volatile unsigned *)(rSCIBase+0x88))
#define rSCITCER     (*(volatile unsigned *)(rSCIBase+0xC0))

/* 3.USB register(8bit)*/
#define rUSB_R0      (*(volatile unsigned *)(rUSBBase+0x00)) /* Function address register */
#define rUSB_R1      (*(volatile unsigned *)(rUSBBase+0x04)) /* Power management register */
#define rUSB_R14     (*(volatile unsigned *)(rUSBBase+0x38)) /* Index register */
#define rUSB_IR1     (*(volatile unsigned *)(rUSBBase+0x40)) /* IN max packet register */
#define rUSB_IR2     (*(volatile unsigned *)(rUSBBase+0x44)) /* IN CSR1 register */
#define rUSB_IR3     (*(volatile unsigned *)(rUSBBase+0x48)) /* IN CSR2 register */
#define rUSB_OR1     (*(volatile unsigned *)(rUSBBase+0x4C)) /* OUT max packet register */
#define rUSB_OR2     (*(volatile unsigned *)(rUSBBase+0x50)) /* OUT CSR1 register */
#define rUSB_OR3     (*(volatile unsigned *)(rUSBBase+0x54)) /* OUT CSR2 register */
#define rEP0_FIFO    (*(volatile unsigned *)(rUSBBase+0x80)) 
#define rEP1_FIFO    (*(volatile unsigned *)(rUSBBase+0x84))
#define rEP2_FIFO    (*(volatile unsigned *)(rUSBBase+0x78))   
#define rUSB_DMAC    (*(volatile unsigned *)(rUSBBase+0xC0)) /* DMA enable/disable 0bit */   

#define USB_R0       (volatile unsigned *)(rUSBBase+0x00)    /* Function address register */
#define USB_R1       (volatile unsigned *)(rUSBBase+0x04)    /* Power management register */
#define USB_R2       (volatile unsigned *)(rUSBBase+0x08)    /* In interrupt register bank1 */
#define USB_R4       (volatile unsigned *)(rUSBBase+0x10)    /* Out interrupt register bank1 */
#define USB_R6       (volatile unsigned *)(rUSBBase+0x18)    /* USB interrpt register */
#define USB_R7       (volatile unsigned *)(rUSBBase+0x1C)    /* In interrupt enable register bank1*/
#define USB_R9       (volatile unsigned *)(rUSBBase+0x24)    /* Out interrupt enable register bank1 */
#define USB_R11      (volatile unsigned *)(rUSBBase+0x2C)    /* USB interrupt enable register */
#define USB_R12      (volatile unsigned *)(rUSBBase+0x30)    /* Frame number1 register */
#define USB_R13      (volatile unsigned *)(rUSBBase+0x34)    /* Frame number2 register */
#define USB_R14      (volatile unsigned *)(rUSBBase+0x38)    /* Index register */
#define USB_IR1      (volatile unsigned *)(rUSBBase+0x40)    /* IN max packet register */
#define USB_IR2      (volatile unsigned *)(rUSBBase+0x44)    /* IN CSR1 register */
#define USB_IR3      (volatile unsigned *)(rUSBBase+0x48)    /* IN CSR2 register */
#define USB_OR1      (volatile unsigned *)(rUSBBase+0x4C)    /* OUT max packet register */
#define USB_OR2      (volatile unsigned *)(rUSBBase+0x50)    /* OUT CSR1 register */
#define USB_OR3      (volatile unsigned *)(rUSBBase+0x54)    /* OUT CSR2 register */
#define EP0_FIFO     (volatile unsigned *)(rUSBBase+0x80) 
#define EP1_FIFO     (volatile unsigned *)(rUSBBase+0x84) 
#define EP2_FIFO     (volatile unsigned *)(rUSBBase+0x88)
#define USB_DMAC     (volatile unsigned *)(rUSBBase+0xC0)    /* DMA enable/disable 0bit */  

/* 4.IEEE 1284(PPI)(8bit)*/
#define rPPDATA      (*(volatile unsigned *)(rPPIBase+0x00)) /* Data register */
#define rPPSTAT	     (*(volatile unsigned *)(rPPIBase+0x04)) /* status register */
#define rPPACKWTH    (*(volatile unsigned *)(rPPIBase+0x08)) /* acknowledge register */
#define rPPCON	     (*(volatile unsigned *)(rPPIBase+0x0C)) /* control register */
#define rPPINTEN     (*(volatile unsigned *)(rPPIBase+0x10)) /* enable interrupt register */
#define rPPINTPND    (*(volatile unsigned *)(rPPIBase+0x14)) /* enterrupt pending register */
#define rPPRLC	     (*(volatile unsigned *)(rPPIBase+0x18)) /* run-length count register */
#define rPPSMR	     (*(volatile unsigned *)(rPPIBase+0x1C)) /* state machine register */

/* 5.IIC register(8bit)*/
#define rIICCON	     (*(volatile unsigned *)(rIICBase+0x00)) /* control register */
#define rIICSTAT     (*(volatile unsigned *)(rIICBase+0x04)) /* status register */
#define rIICADD	     (*(volatile unsigned *)(rIICBase+0x08)) /* address register */
#define rIICDATA     (*(volatile unsigned *)(rIICBase+0x0C)) /* data register */

/* 6.TIMER register(8bit)*/
#define rTCON0       (*(volatile unsigned *)(rTIMEBase+0x00))
#define rTCON1       (*(volatile unsigned *)(rTIMEBase+0x04))
#define rTCON2       (*(volatile unsigned *)(rTIMEBase+0x08))
#define rTCON3       (*(volatile unsigned *)(rTIMEBase+0x0C))
#define rTPSR0       (*(volatile unsigned *)(rTIMEBase+0x10))
#define rTPSR1       (*(volatile unsigned *)(rTIMEBase+0x14))
#define rTPSR2       (*(volatile unsigned *)(rTIMEBase+0x18))
#define rTPSR3       (*(volatile unsigned *)(rTIMEBase+0x1C))
#define rTDR0        (*(volatile unsigned *)(rTIMEBase+0x20))
#define rTDR1        (*(volatile unsigned *)(rTIMEBase+0x24))
#define rTDR2        (*(volatile unsigned *)(rTIMEBase+0x28))
#define rTDR3        (*(volatile unsigned *)(rTIMEBase+0x2C))
#define rTCR0        (*(volatile unsigned *)(rTIMEBase+0x30))
#define rTCR1        (*(volatile unsigned *)(rTIMEBase+0x34))
#define rTCR2        (*(volatile unsigned *)(rTIMEBase+0x38))
#define rTCR3        (*(volatile unsigned *)(rTIMEBase+0x3C))

/* 7.RTC register(8bit)*/
#define rRTCCON	     (*(volatile unsigned *)(rRTCBase+0x00)) /* RTC control register */
#define rRTCRST	     (*(volatile unsigned *)(rRTCBase+0x04)) /* RTC round reset register */
#define rRTCALM	     (*(volatile unsigned *)(rRTCBase+0x08)) /* RTC alarm register */
#define rALMSEC	     (*(volatile unsigned *)(rRTCBase+0x0C)) /* Alarm second data register */
#define rALMMIN	     (*(volatile unsigned *)(rRTCBase+0x10))
#define rALMHOUR     (*(volatile unsigned *)(rRTCBase+0x14)) /* Alarm hour data register */
#define rALMDATE     (*(volatile unsigned *)(rRTCBase+0x18)) /* Alarm date data register */
#define rALMDAY	     (*(volatile unsigned *)(rRTCBase+0x1C)) /* Alarm day data register */
#define rALMMON	     (*(volatile unsigned *)(rRTCBase+0x20)) /* Alarm mon data register */
#define rALMYEAR     (*(volatile unsigned *)(rRTCBase+0x24)) /* Alarm year data register */
#define rBCDSEC	     (*(volatile unsigned *)(rRTCBase+0x28)) /* BCD second data register */
#define rBCDMIN	     (*(volatile unsigned *)(rRTCBase+0x2C)) /* BCD minute data register */
#define rBCDHOUR     (*(volatile unsigned *)(rRTCBase+0x30)) /* BCD hour data register */
#define rBCDDATE     (*(volatile unsigned *)(rRTCBase+0x34)) /* BCD day data register */
#define rBCDDAY	     (*(volatile unsigned *)(rRTCBase+0x38)) /* BCD day data register */
#define rBCDMON	     (*(volatile unsigned *)(rRTCBase+0x3C)) /* BCD month data register */
#define rBCDYEAR     (*(volatile unsigned *)(rRTCBase+0x40)) /* BCD year  data register */
#define rRTCIM       (*(volatile unsigned *)(rRTCBase+0x44)) /* BCD year  data register */
#define rRTCPEND     (*(volatile unsigned *)(rRTCBase+0x48)) /* BCD year  data register */

/* 8.Watch Dog Timer register(8bit)*/
#define rWDTMOD	     (*(volatile unsigned *)(rWDTBase+0x00)) /* Watch dog timer mode register */
#define rWDTDAT	     (*(volatile unsigned *)(rWDTBase+0x04)) /* Watch dog timer data register */
#define rWDTCNT	     (*(volatile unsigned *)(rWDTBase+0x08)) /* Watch dog timer count register */

/* 9.Programmable I/O port */
#define rGIOPCON     (*(volatile unsigned *)(rIOPBase+0x00)) /* Port direction register */
#define rGIOPDATA    (*(volatile unsigned *)(rIOPBase+0x04)) /* Data register */
#define rGIOPINTEN   (*(volatile unsigned *)(rIOPBase+0x08)) /* Interrupt enable register */
#define rGIOPLEVEL   (*(volatile unsigned *)(rIOPBase+0x0C)) /* Ative level indication register */
#define rGIOPPEND    (*(volatile unsigned *)(rIOPBase+0x10)) /* Interrupt pending register */

/* 10.Interrupt controller */
#define rINTCON      (*(volatile unsigned *)(rINTBase+0x00)) /* interrupt control register */
#define rINTPND	     (*(volatile unsigned *)(rINTBase+0x04)) /* interrupt pending register */
#define rINTMOD	     (*(volatile unsigned *)(rINTBase+0x08)) /* interrupt mode register */
#define rINTMSK	     (*(volatile unsigned *)(rINTBase+0x0C)) /* interrupt mask register */
#define rINTLEVEL    (*(volatile unsigned *)(rINTBase+0x10)) 
#define rIRQPSLV0    (*(volatile unsigned *)(rINTBase+0x14)) /* IRQ priority of slave register0 */
#define rIRQPSLV1    (*(volatile unsigned *)(rINTBase+0x18)) /* IRQ priority of slave register1 */
#define rIRQPSLV2    (*(volatile unsigned *)(rINTBase+0x1C)) /* IRQ priority of slave register2 */
#define rIRQPSLV3    (*(volatile unsigned *)(rINTBase+0x20)) /* IRQ priority of slave register3 */
#define rIRQPMST     (*(volatile unsigned *)(rINTBase+0x24)) /* IRQ priority of master register */
#define rIRQCSLV0    (*(volatile unsigned *)(rINTBase+0x28)) /* current IRQ priority of slave register0 */
#define rIRQCSLV1    (*(volatile unsigned *)(rINTBase+0x2C)) /* current IRQ priority of slave register1 */
#define rIRQCSLV2    (*(volatile unsigned *)(rINTBase+0x30)) /* current IRQ priority of slave register2 */
#define rIRQCSLV3    (*(volatile unsigned *)(rINTBase+0x34)) /* current IRQ priority of slave register3 */
#define rIRQCMST     (*(volatile unsigned *)(rINTBase+0x38)) /* current IRQ priority of master register */
#define rIRQISPR     (*(volatile unsigned *)(rINTBase+0x3C)) /* IRQ service pending register */
#define rIRQISPC     (*(volatile unsigned *)(rINTBase+0x40)) /* IRQ service clear register */
#define rFIQPSLV0    (*(volatile unsigned *)(rINTBase+0x44)) /* FIQ priority of slave register0 */
#define rFIQPSLV1    (*(volatile unsigned *)(rINTBase+0x48)) /* FIQ priority of slave register1 */
#define rFIQPSLV2    (*(volatile unsigned *)(rINTBase+0x4C)) /* FIQ priority of slave register2 */
#define rFIQPSLV3    (*(volatile unsigned *)(rINTBase+0x50)) /* FIQ priority of slave register3 */
#define rFIQPMST     (*(volatile unsigned *)(rINTBase+0x54)) /* FIQ priority of master register */
#define rFIQCSLV0    (*(volatile unsigned *)(rINTBase+0x58)) /* current FIQ priority of slave register0 */
#define rFIQCSLV1    (*(volatile unsigned *)(rINTBase+0x5C)) /* current FIQ priority of slave register1 */
#define rFIQCSLV2    (*(volatile unsigned *)(rINTBase+0x60)) /* current FIQ priority of slave register2 */
#define rFIQCSLV3    (*(volatile unsigned *)(rINTBase+0x64)) /* current FIQ priority of slave register3 */
#define rFIQCMST     (*(volatile unsigned *)(rINTBase+0x68)) /* current FIQ priority of master register */
#define rFIQISPR     (*(volatile unsigned *)(rINTBase+0x6C)) /* FIQ service pending register */
#define rFIQISPC     (*(volatile unsigned *)(rINTBase+0x70)) /* FIQ service clear register */
#define rPOLARITY    (*(volatile unsigned *)(rINTBase+0x74))
#define rIVEC_ADDR   (*(volatile unsigned *)(rINTBase+0x78))
#define rFVEC_ADDR   (*(volatile unsigned *)(rINTBase+0x78))

/* 11.SSP(prime cell) Synchronous serial port register(16bit) */
#define rSSPCR0      (*(volatile unsigned *)(rSSPBase+0x00)) /* SSP control0 register */
#define rSSPCR1      (*(volatile unsigned *)(rSSPBase+0x04)) /* SSP control1 register */
#define rSSPDR       (*(volatile unsigned *)(rSSPBase+0x08)) /* SSP data register */
#define rSSPSR       (*(volatile unsigned *)(rSSPBase+0x0C)) /* SSP status register */
#define rSSPCPSR     (*(volatile unsigned *)(rSSPBase+0x10)) /* SSP clock prescale register */
#define rSSPIIR      (*(volatile unsigned *)(rSSPBase+0x14)) /* SSP interrupt identification register */

/* 12.KMI0(prime cell) Keyboard/Mouse interface register(8bit) */
#define rKMI0CR       (*(volatile unsigned *)(rKMI0Base+0x00)) /* KMI control register */
#define rKMI0STAT     (*(volatile unsigned *)(rKMI0Base+0x04)) /* KMI status register */
#define rKMI0DATA     (*(volatile unsigned *)(rKMI0Base+0x08)) /* Tx/Rx data register */
#define rKMI0CLKDIV   (*(volatile unsigned *)(rKMI0Base+0x0C)) /* Clock divisor register */
#define rKMI0IIR      (*(volatile unsigned *)(rKMI0Base+0x10)) /* KMI interrupt register */

/* 12.KMI0(prime cell) Keyboard/Mouse interface register(8bit) */
#define rKMI1CR       (*(volatile unsigned *)(rKMI1Base+0x00)) /* KMI control register */
#define rKMI1STAT     (*(volatile unsigned *)(rKMI1Base+0x04)) /* KMI status register */
#define rKMI1DATA     (*(volatile unsigned *)(rKMI1Base+0x08)) /* Tx/Rx data register */
#define rKMI1CLKDIV   (*(volatile unsigned *)(rKMI1Base+0x0C)) /* Clock divisor register */
#define rKMI1IIR      (*(volatile unsigned *)(rKMI1Base+0x10)) /* KMI interrupt register */

/* 13.UART0(prime cell) register(8bit)*/
#define rUART0DR     (*(volatile unsigned *)(rUART0Base+0x00)) /* Rx\Tx buffer register */
#define rUART0RSR    (*(volatile unsigned *)(rUART0Base+0x04)) /* Receive status register */
#define rUART0LCR_H  (*(volatile unsigned *)(rUART0Base+0x08)) /* Line control register */
#define rUART0LCR_M  (*(volatile unsigned *)(rUART0Base+0x0C)) /* Baud rate high register */
#define rUART0LCR_L  (*(volatile unsigned *)(rUART0Base+0x10)) /* Baud rate low register */
#define rUART0CR     (*(volatile unsigned *)(rUART0Base+0x14)) /* Conrol register */
#define rUART0FR     (*(volatile unsigned *)(rUART0Base+0x18)) /* Flag register */
#define rUART0IIR    (*(volatile unsigned *)(rUART0Base+0x1C)) /* Interrupt identification register */
#define rUART0ILPR   (*(volatile unsigned *)(rUART0Base+0x20)) /* IrDA low power register */
#define rUART0_DMAC  (*(volatile unsigned *)(rUART0Base+0x24)) /* UART0 DMA Rx(0),Tx(1) enable/disable */
#define rUART0TMR    (*(volatile unsigned *)(rUART0Base+0x84)) 

/* 13.UART1(prime cell) register(8bit)*/
#define rUART1DR     (*(volatile unsigned *)(rUART1Base+0x00)) /* Rx\Tx buffer register */
#define rUART1RSR    (*(volatile unsigned *)(rUART1Base+0x04)) /* Receive status register */
#define rUART1LCR_H  (*(volatile unsigned *)(rUART1Base+0x08)) /* Line control register */
#define rUART1LCR_M  (*(volatile unsigned *)(rUART1Base+0x0C)) /* Baud rate high register */
#define rUART1LCR_L  (*(volatile unsigned *)(rUART1Base+0x10)) /* Baud rate low register */
#define rUART1CR     (*(volatile unsigned *)(rUART1Base+0x14)) /* Conrol register */
#define rUART1FR     (*(volatile unsigned *)(rUART1Base+0x18)) /* Flag register */
#define rUART1IIR    (*(volatile unsigned *)(rUART1Base+0x1C)) /* Interrupt identification register */
#define rUART1ILPR   (*(volatile unsigned *)(rUART1Base+0x20)) /* IrDA low power register */
#define rUART1_DMAC  (*(volatile unsigned *)(rUART1Base+0x24)) /* UART1 DMA Rx(0),Tx(1) enable/disable */
#define rUART1TMR    (*(volatile unsigned *)(rUART1Base+0x84)) 

/* 15.Power manager register */ 
#define rPLLCON      (*(volatile unsigned *)(rPMBase+0x00)) /* pll configuration register */
#define rMODCON      (*(volatile unsigned *)(rPMBase+0x04)) /* mode control register */
#define rHCLKCON     (*(volatile unsigned *)(rPMBase+0x08)) /* AHB clock control register */
#define rPCLKCON     (*(volatile unsigned *)(rPMBase+0x0C)) /* APB clock control register */

/* 16.Switch box register */
#define rSBCON0      (*(volatile unsigned *)(rMISCBase+0x00)) /* Switch box control0 register */
#define rSBCON1      (*(volatile unsigned *)(rMISCBase+0x04)) /* Switch box control1 register */

/* 17.Teak base register */
#define rTeakArmCtrl (*(volatile unsigned *)(rTEAKBase+0x7F0)) /* Control register */
#define rTeakArmMsg0 (*(volatile unsigned *)(rTEAKBase+0x7F4)) /* Message register0 */
#define rTeakArmMsg1 (*(volatile unsigned *)(rTEAKBase+0x7F8)) /* Message register1 */
#define rTeakArmMsg2 (*(volatile unsigned *)(rTEAKBase+0x7FC)) /* Message register2 */
