/*********************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : May. 2004

 File          : XIRQ0.c

 Hardware      : Applicable to ADuC7020/ADuC7024/ADuC7026
                 Currently targetting the ADuC7024

 Description   : Complement port 3 in a loop.
		 When XIRQ0 is pressed, complement P4.2	

		 
 Notes         : To debug this code, either the IAR ROM Monitor or the
                 J-Link must be used. The linker script currently used 
                 (lnkARM_ADIROM.xcl) compiles the code for RAM. To 
                 compile for Flash/EE, change the linker command file under
                 "Project Options->Linker->Config->Linker Command File" 
                 to the Flash/EE linker script (LinkADIFlash.xcl) and 
                 download via WSD.   
                 
                 This project uses a modified version of cstartup.s79 which
                 enables IRQ interrupts. 
                 
                 Note set the baud rate for the Rom Monitor to 9600baud.             	 
                 
                  
*********************************************************************/

#include "ioaduc7024.h"					// 	Include ADuC7024 Header File
                                                        //      required for interrupts


__irq __arm void irq_handler(void);			//	IRQ Function Prototype 


void delay(int);

 

__irq __arm void irq_handler(void)                      //      IRQ Handler
{
	GP4DAT ^= 0x04040000;				//      Complement P4.2
	while(GP0DAT & 0x00010){}			//      wait for XIRQ to be low again
	return ;
}



int main (void)  
{

         
	IRQEN = XIRQ0_BIT;				//	Enable XIRQ0 in IRQEnable 

	GP0CON = 0x00;
	GP4DAT = 0x04000000;				//	Configure P4.2 as output
	GP3DAT = 0xff000000;				//	Configure port 3 as output

	while(1)
	{
		GP3DAT ^= 0x00ff0000;	   		// 	Complement Port 3
		delay(900000);				//	Delay to make blink visible
	}
}



void delay (int length)                                 //      Delay function
{
	while (length >=0)
    	length--;
}


