/* fflush function */
#include "xstdio.h"
#include "yfuns.h"

#if _DLIB_FILE_DESCRIPTOR

_STD_BEGIN

int (fflush)(FILE *str)
	{	/* flush an output stream */
	size_t n;
	unsigned char *s;
        int report_error = 0;

	if (str == 0)
		{	/* recurse on all streams */
		int nf, stat;

		_Locksyslock(_LOCK_STREAM);
		for (stat = 0, nf = 0; nf < FOPEN_MAX; ++nf)
			if (_Files[nf] != 0 && _CSTD fflush(_Files[nf]) < 0)
				stat = EOF;
		_Unlocksyslock(_LOCK_STREAM);
		return (stat);
		}

	_Lockfileatomic(str);
	if (!(str->_Mode & _MWRITE))
		{	/* stream not writable */
		_Unlockfileatomic(str);
		return (0);
		}
	for (s = str->_Buf; s < str->_Next; s += n)
		{	/* try to write buffer */
		n = __write(str->_Handle, s, str->_Next - s);
		if (n == _LLIO_ERROR || n == 0)
			{	/* report error and fail */
                        report_error = 1;
                        break;
			}
		}
        /* Flush low-level I/O, if it also is buffered. */
        if (   report_error
            || __write(str->_Handle, 0, 0) == _LLIO_ERROR)
                {
                str->_Next = str->_Buf;
                str->_Wend = str->_Buf, str->_WWend = str->_Buf;
                str->_Mode |= _MERR;
                _Unlockfileatomic(str);
                return (EOF);
                }

	str->_Next = str->_Buf;
	if ((str->_Mode & (_MNBF | _MLBF)) != 0)
		s = str->_Buf;
	else
		s = str->_Bend;
	if (str->_Mode & _MBYTE)
		str->_Wend = s;
	else
		str->_WWend = s;
	_Unlockfileatomic(str);
	return (0);
	}
_STD_END

#endif /* _DLIB_FILE_DESCRIPTOR */

/*
 * Copyright (c) 1992-2002 by P.J. Plauger.  ALL RIGHTS RESERVED.
 * Consult your license regarding permissions and restrictions.
V3.12:0576 */
