// fstream implementation.
#include <yvals.h>

#if _DLIB_FILE_DESCRIPTOR

#include <fstream>

_STD_BEGIN

filebuf::_Myt *filebuf::open(const char *_Filename, ios_base::openmode _Mode,
                             int _Prot)
{       // open a C stream with specified mode
  _Filet *_File;
  if (_Myfile != 0 || (_File = _Fiopen(_Filename, _Mode, _Prot)) == 0)
    return (0);     // open failed

  _Init(_File, _Openfl);
  return (this);  // open succeeded
}

filebuf::_Myt *filebuf::open(const char *_Filename, ios_base::open_mode _Mode)
{       // open a C stream with specified mode (old style)
  return (open(_Filename, (ios_base::openmode)_Mode));
}

filebuf::_Myt *filebuf::close()
{       // close the C stream
  if (_Myfile != 0 && _Endwrite() && fclose(_Myfile) == 0)
  {       // close succeeded, tidy up
    _Init(0, _Closefl);
    return (this);
  }
  else
    return (0);
}

filebuf::int_type filebuf::overflow(int_type _Meta)
{       // put an element to stream
  if (_Traits::eq_int_type(_Traits::eof(), _Meta))
    return (_Traits::not_eof(_Meta));       // EOF, return success code
  else if (_Mysb::pptr() != 0
           && _Mysb::pptr() < _Mysb::epptr())
  {       // room in buffer, store it
    *_Mysb::_Pninc() = _Traits::to_char_type(_Meta);
    return (_Meta);
  }
  else if (_Myfile == 0)
    return (_Traits::eof());        // no open C stream, fail
  else
    return (fputc(_Traits::to_char_type(_Meta), _Myfile) != EOF
            ? _Meta : _Traits::eof());      // put the char
}

filebuf::int_type filebuf::pbackfail(int_type _Meta)
{       // put an element back to stream
  if (_Mysb::gptr() != 0
      && _Mysb::eback() < _Mysb::gptr()
      && (_Traits::eq_int_type(_Traits::eof(), _Meta)
          || _Traits::eq_int_type(_Traits::to_int_type(_Mysb::gptr()[-1]),
                                  _Meta)))
  {       // just back up position
    _Mysb::_Gndec();
    return (_Traits::not_eof(_Meta));
  }
  else if (_Myfile == 0 || _Traits::eq_int_type(_Traits::eof(), _Meta))
    return (_Traits::eof());        // no open C stream or EOF, fail
  else
    return (ungetc(_Traits::to_char_type(_Meta), _Myfile) != EOF
            ? _Meta : _Traits::eof());      // put back the char
}

filebuf::int_type filebuf::underflow()
{       // get an element from stream, but don't point past it
  int_type _Meta;
  if (_Mysb::gptr() != 0
      && _Mysb::gptr() < _Mysb::egptr())
    return (_Traits::to_int_type(*_Mysb::gptr()));  // return buffered
  else if (_Traits::eq_int_type(_Traits::eof(), _Meta = uflow()))
    return (_Meta); // uflow failed, return EOF
  else
  {       // get a char, don't point past it
    pbackfail(_Meta);
    return (_Meta);
  }
}

filebuf::int_type filebuf::uflow()
{       // get an element from stream, point past it
  if (_Mysb::gptr() != 0
      && _Mysb::gptr() < _Mysb::egptr())
    return (_Traits::to_int_type(
              *_Mysb::_Gninc()));     // return buffered
  else if (_Myfile == 0)
    return (_Traits::eof());        // no open C stream, fail
  else
  {       // get the char
    int_type _Byte;
    return ((_Byte = fgetc(_Myfile)) != EOF ? _Byte
            : _Traits::eof());
  }
}

filebuf::pos_type filebuf::seekoff(off_type _Off, ios_base::seekdir _Way,
                                   ios_base::openmode)
{       // change position by _Off
  fpos_t _Fileposition;

  if (_Myfile == 0 || !_Endwrite()
      || (_Off != 0 || _Way != ios_base::cur)
      && fseek(_Myfile, (long)_Off, _Way) != 0
      || fgetpos(_Myfile, &_Fileposition) != 0)
    return (pos_type(_BADOFF));     // report failure
  return (_POS_TYPE_FROM_STATE(pos_type, _State,
                               _Fileposition));        // return new position
}

filebuf::pos_type filebuf::seekpos(pos_type _Pos,
                                   ios_base::openmode)
{       // change position to _Pos
  fpos_t _Fileposition = _POS_TYPE_TO_FPOS_T(_Pos);
  off_type _Off = (off_type)_Pos - _FPOSOFF(_Fileposition);

  if (_Myfile == 0 || !_Endwrite()
      || fsetpos(_Myfile, &_Fileposition) != 0
      || _Off != 0 && fseek(_Myfile, (long)_Off, SEEK_CUR) != 0
      || fgetpos(_Myfile, &_Fileposition) != 0)
    return (pos_type(_BADOFF));     // report failure
  return (_POS_TYPE_FROM_STATE(pos_type, _State,
                               _Fileposition));        // return new position
}

filebuf::_Mysb *filebuf::setbuf(_Elem *_Buffer, streamsize _Count)
{       // offer _Buffer to C stream
  return (_Myfile == 0 || setvbuf(_Myfile, (char *)_Buffer,
                                  _Buffer == 0 && _Count == 0 ? _IONBF : _IOFBF,
                                  _Count * sizeof (_Elem)) != 0 ? 0 : this);
}

int filebuf::sync()
{       // synchronize C stream with external file
  return (_Myfile == 0
          || _Traits::eq_int_type(_Traits::eof(), overflow())
          || 0 <= fflush(_Myfile) ? 0 : -1);
}

void filebuf::_Init(_Filet *_File, _Initfl _Which)
{       // initialize to C stream _File after {new, open, close}
  static _Traits::state_type _Stinit;     // initial state
  _Closef = _Which == _Openfl;

  _Mysb::_Init(); // initialize stream buffer base object

  if (_File != 0 && sizeof (_Elem) == 1)
    _Mysb::_Init((_Elem **)&_File->_Buf,
                 (_Elem **)&_File->_Next,
                 (_Elem **)&_File->_Rend,
                 (_Elem **)&_File->_Buf,
                 (_Elem **)&_File->_Next,
                 (_Elem **)&_File->_Wend);
  _Myfile = _File;
  _State = _Stinit;
}

_STD_END

#endif /* _DLIB_FILE_DESCRIPTOR */

/*
 * Copyright (c) 1992-2002 by P.J. Plauger.  ALL RIGHTS RESERVED.
 * Consult your license regarding permissions and restrictions.
V3.12:0576 */
