/**************************************************
 *
 * Segment initialization that must be
 * performed before main is called.
 *
 * Since this function is called from cstartup,
 * it is located in the same segment, ICODE.
 *
 * Copyright 1999-2004 IAR Systems. All rights reserved.
 *
 * $Revision: 1.6.1.2 $
 *
 **************************************************/

#include "segment_init.h"
#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif

#pragma language=extended
#pragma segment="INITTAB"

#pragma location="ICODE"
__interwork void __segment_init(void)
{
  InitBlock_Type const * const initTableBegin = __sfb( "INITTAB" );
  InitBlock_Type const * const initTableEnd = __sfe( "INITTAB" );
  InitBlock_Type const * initTableP;

  /* Loop over all elements in the initialization table. */
  for (initTableP=initTableBegin; initTableP<initTableEnd; initTableP++)
  {
    /* If src=dest then we should clear a memory
     * block, otherwise it's a copy operation. */
    if (initTableP->Src == initTableP->Dst)
    {
      memset(initTableP->Dst, 0, initTableP->Size);
    }
    else
    {
      memcpy(initTableP->Dst, initTableP->Src, initTableP->Size);
    }
  }
}

#pragma language=default

#ifdef __cplusplus
}
#endif
