/******************************************************************************
*                                                             \  ___  /       *
*                                                               /   \         *
* Edison Design Group C++ Runtime                            - | \^/ | -      *
*                                                               \   /         *
* Proprietary information of Edison Design Group Inc.         /  | |  \       *
* Copyright 1992-1996 Edison Design Group Inc.                   [_]          *
*                                                                             *
******************************************************************************/
/*

C++ runtime routines to provide vector new() and delete() functionality.

*/

#ifdef __IAR_SYSTEMS_ICC__
#ifndef __cplusplus
#error Compile this file in EC++ mode.
#endif
#endif

#include "basics.h"
#include "runtime.h"
#if EXCEPTION_HANDLING
#include "eh.h"
#endif
#pragma hdrstop
#include "vec_newdel.h"
#include "main.h"

#pragma language = extended

/*
For arrays, _vec_new() and _vec_delete() will maintain a linked list of 
"hidden" information on each array allocated and subsequently deleted.
This information will be used by _vec_ctor() and _vec_dtor() to determine
the size of undimensioned arrays.
*/

#if ABI_CHANGES_FOR_ARRAY_NEW_AND_DELETE
#define default_array_operator_new(M) operator new[] M
#define default_array_operator_delete operator delete[]
#else /* !ABI_CHANGES_FOR_ARRAY_NEW_AND_DELETE */
#define default_array_operator_new(M) operator new M
#define default_array_operator_delete operator delete
#endif /* ABI_CHANGES_FOR_ARRAY_NEW_AND_DELETE */

#if !USE_PREFIX_FOR_ARRAY_ALLOC_INFO
#error "Unsupported configuration (!USE_PREFIX_FOR_ARRAY_ALLOC_INFO)"
#endif /* !USE_PREFIX_FOR_ARRAY_ALLOC_INFO */

#if CFRONT_COMPATIBILITY_MODE
#error "Unsupported configuration (CFRONT_COMPATIBILITY_MODE)"
#endif /* CFRONT_COMPATIBILITY_MODE */

EXTERN_C void _array_pointer_not_from_vec_new();
                               /* Function called when an invalid pointer that
                                  was not allocated by vec_new is passed
                                  to one of the vector handling routines. */


#if EXCEPTION_HANDLING
#error "This configuration (!EXCEPTION_HANDLING) not updated " \
       "for multiple class memory support"
struct an_array_alloc_eh_info {
  void* array_ptr;
                        /* Pointer to the memory allocated for the array. */
  a_sizeof_t
                number_of_elements;
                        /* Total number of elements in the array. */
  a_sizeof_t
                element_size;
                        /* Size of each element. */
  a_sizeof_t
                elements_processed;
                        /* Number of elements constructed or destructed so
                           far. */
  a_boolean
                is_vec_new;
                        /* TRUE if this is a vec_new operation, FALSE if this
                           is a vec_delete. */
  a_boolean
                free_memory_on_cleanup;
                        /* TRUE if the memory for the array was allocated by
                           new and should be freed during object cleanup. */
  a_destructor_ptr
                destructor;
                        /* Pointer to the destructor to be called for each
                           element of the array. */
  a_delete_ptr
                delete_routine;
                        /* Pointer to the delete routine to be called to
                           deallocate an array. */

  a_boolean     is_two_arg;
                        /* TRUE if the delete routine is the two argument
                           form.  FALSE if it is the single argument form. */
};
#endif /* EXCEPTION_HANDLING */


/*
Increment a void* pointer by a given value.
*/
#define increment_ptr(M, ptr, incr) (ptr = ((char M *)ptr + incr))

#if EXCEPTION_HANDLING
static void add_vec_new_or_delete_eh_stack_entry
                                (an_eh_stack_entry_ptr      ehsep,
                                 an_array_alloc_eh_info_ptr aaehip,
                                 a_boolean                  is_vec_new)
/*
Link an entry onto the EH stack that describes the vec_new or vec_delete
operation that is in process.
*/
{
  ehsep->next = __curr_eh_stack_entry;
  __curr_eh_stack_entry = ehsep;
  ehsep->kind = ehsek_vec_new_or_delete;
  ehsep->variant.array_alloc_eh_info = aaehip;
  aaehip->array_ptr                 = NULL;
  aaehip->number_of_elements        = 0;
  aaehip->element_size              = 0;
  aaehip->elements_processed        = 0;
  aaehip->is_vec_new                = is_vec_new;
  aaehip->free_memory_on_cleanup    = FALSE;
  aaehip->destructor                = NULL;
  aaehip->delete_routine            = NULL;
  aaehip->is_two_arg                = FALSE;
}  /* add_vec_new_or_delete_eh_stack_entry */
#endif /* EXCEPTION_HANDLING */


/*
Structure used to record the size of an array that has been allocated.
Space for this structure is reserved at the beginning of the block of
memory allocated to hold the array.
*/
#define __VAR_MEM_HELPER1__(M, I)                                             \
typedef struct M##_alloc_prefix M *M##_alloc_prefix_ptr;                      \
struct M##_alloc_prefix {                                                     \
  M##_size_t    size;                                                         \
                        /* The size of the array, in bytes. */                \
  M##_size_t    encoded_number_of_elements;                                   \
                        /* The number of elements in the array.  This value   \
                           is encoded so that a zero value is not represented \
                           as a zero.  This is done to help detect situations \
                           in which the prefix has been overwritten. */       \
};                                                                            \
                                                                              \
/*                                                                            \
Compute the size in bytes of the prefix to be allocated.  This must           \
be at least as large as an_alloc_prefix, but must also be a multiple          \
of the most strict alignment. */                                              \
extern const M##_size_t  M##_array_new_prefix_size =                          \
                  ((sizeof(M##_alloc_prefix) + MOST_STRICT_ALIGNMENT - 1) /   \
                              MOST_STRICT_ALIGNMENT) * MOST_STRICT_ALIGNMENT;
__VAR_MEMORY_LIST1__()
#undef __VAR_MEM_HELPER1__


#define __VAR_MEM_HELPER1__(M, I)                                             \
static inline void M * M##_alloc_array(M##_size_t          size,              \
                                       M##_new_ptr       new_routine)         \
/*                                                                            \
Call the allocation routine to allocate the memory for the array.  Adjust     \
the size as needed to provide storage for the prefix information used to      \
save the array size.                                                          \
*/                                                                            \
{                                                                             \
  void M        *array_ptr;                                                   \
  /* Increment the size to provide space for the prefix block. */             \
  size += M##_array_new_prefix_size;                                          \
  array_ptr = (*new_routine)(size);                                           \
  if (array_ptr != NULL) {                                                    \
    /* Return a pointer to the part of the array after the prefix. */         \
    array_ptr = ((char M *)array_ptr) + M##_array_new_prefix_size;            \
  }  /* if */                                                                 \
  return array_ptr;                                                           \
}  /* alloc_array */
__VAR_MEMORY_LIST1__()
#undef __VAR_MEM_HELPER1__


#define __VAR_MEM_HELPER1__(M, I)                                             \
static void M##_free_array(void M *         array_ptr,                        \
                           M##_size_t       size,                             \
                           M##_delete_ptr   delete_routine,                   \
                           int              is_two_arg)                       \
/*                                                                            \
Call the deallocation routine to free the memory for the array.  Adjust       \
the size as needed to provide storage for the prefix information used to      \
save the array size.                                                          \
*/                                                                            \
{                                                                             \
  /* Increment the size to provide space for the prefix block. */             \
  size += M##_array_new_prefix_size;                                          \
  /* Adjust the pointer to point to the start of the prefix. */               \
  array_ptr = ((char M *)array_ptr) - M##_array_new_prefix_size;              \
  if (is_two_arg) {                                                           \
    M##_two_operand_delete_ptr two_op_delete_routine;                         \
    two_op_delete_routine = (M##_two_operand_delete_ptr)delete_routine;       \
    (*two_op_delete_routine)(array_ptr, size);                                \
  } else {                                                                    \
    (*delete_routine)(array_ptr);                                             \
  }  /* if */                                                                 \
}  /* free_array */
__VAR_MEMORY_LIST1__()
#undef __VAR_MEM_HELPER1__


#define __VAR_MEM_HELPER1__(M, I)                                             \
static inline a_boolean M##_record_array_alloc_info(                          \
                                         void M *     array_ptr,              \
                                         M##_size_t   size,                   \
                                         M##_size_t   number_of_elements)     \
/*                                                                            \
Record the size of the array so that it can be retrieved later using          \
the array pointer.  Returns TRUE if an error occurred and the size            \
could not be recorded.                                                        \
*/                                                                            \
{                                                                             \
  M##_alloc_prefix_ptr   app;                                                 \
  /* Get a pointer to the prefix information and fill in the fields.          \
     The number of elements is saved as the compliment of the actual          \
     value.  This is done so that zeroing out the prefix (as might happen     \
     if the memory were overwritten) would not result in valid values. */     \
  app = (M##_alloc_prefix_ptr)                                                \
                       (((char M *)array_ptr) - M##_array_new_prefix_size);   \
  app->size = size;                                                           \
  app->encoded_number_of_elements = ~number_of_elements;                      \
  return FALSE;                                                               \
}  /* record_array_alloc_info */
__VAR_MEMORY_LIST1__()
#undef __VAR_MEM_HELPER1__


#define __VAR_MEM_HELPER1__(M, I)                                             \
static inline M##_size_t M##_get_array_size(void M *   array_ptr,             \
                                            M##_size_t element_size)          \
{                                                                             \
  M##_alloc_prefix_ptr   app;                                                 \
  M##_size_t             size;                                                \
  M##_size_t             size_to_check;                                       \
  /* Get the size from the prefix.  Compute the size from the                 \
     element count saved in the prefix and the element size passed            \
     by the caller.  Note that the number of elements is saved as the         \
     compliment of the actual value.  If the two sizes do not agree,          \
     either the memory was not allocated by array_new, or the prefix was      \
     corrupted. */                                                            \
  app = (M##_alloc_prefix_ptr)(((char M *)array_ptr)                          \
                                 - M##_array_new_prefix_size);                \
  size = app->size;                                                           \
  size_to_check = element_size * (~(app->encoded_number_of_elements));        \
  if (size != size_to_check) _array_pointer_not_from_vec_new();               \
  return size;                                                                \
}  /* get_array_size */
__VAR_MEMORY_LIST1__()
#undef __VAR_MEM_HELPER1__


/* Do this function twice, once with EXCEPTION_HANDLING, once without. */
#if EXCEPTION_HANDLING
#define __VAR_MEM_HELPER1__(M, I)                                             \
static void M *M##_array_new_general(void M *            array_ptr,           \
                                     M##_size_t          number_of_elements,  \
                                     M##_size_t          element_size,        \
                                     void M *            src_array_ptr,       \
                                     M##_constructor_ptr ctor,                \
                                     M##_destructor_ptr  dtor,                \
                                     M##_new_ptr         new_routine,         \
                                     M##_delete_ptr      delete_routine,      \
                                     int                 is_two_arg,          \
                                     a_boolean           record_array_info)   \
/*                                                                            \
Allocate storage for an array, then call a constructor for each               \
element of the array.  If array_ptr is NULL, allocate the space for an        \
array of class objects (with number_of_elements elements each of size         \
element_size).  Also remember the size of the array in a                      \
behind-the-scenes data structure so that it can be recalled at the            \
time of the corresponding vec_delete call.  If array_ptr is non-NULL,         \
it points to an already-allocated array.  If ctor is non-NULL, it             \
points to a constructor function to be called for each element of the         \
array (whether the array is allocated here or pre-allocated).  Return         \
the address of the array.                                                     \
                                                                              \
src_array_ptr points to the array to be copied when ctor points to a          \
copy constructor.                                                             \
                                                                              \
dtor is a pointer to the destructor for objects of the element type.          \
This is used by the exception handling mechanism for object cleanup           \
if an exception is thrown while the array is being constructed.               \
If there is no destructor then dtor is NULL and no cleanup is done.           \
                                                                              \
delete_routine is a pointer to the delete routine to be used to deallocate    \
the space in the event that an exception is thrown during construction.       \
is_two_arg is TRUE if delete_routine refers to a two argument version         \
of the delete operator.  record_array_info is TRUE if the array size          \
information should be saved even though an array_ptr value was provided.      \
                                                                              \
This routine needs to record the size of the array that was allocated so      \
that the size is known when the array is deallocated.  One of two means       \
of recording this information is used depending on the setting of the         \
configuration flag USE_PREFIX_FOR_ARRAY_ALLOC_INFO.  If this flag is TRUE,    \
a prefix structure is allocated as part of the array allocation, and this     \
prefix is used to store the size.  The pointer returned to the caller points  \
to the memory after the prefix block.                                         \
                                                                              \
When USE_PREFIX_FOR_ARRAY_ALLOC_INFO is FALSE, a linked list is used          \
to record the number of elements in the array.  Consequently, the             \
performance degrades if a large number of arrays are allocated.  For          \
this reason, the alternate mechanism is recommended for production            \
use.                                                                          \
*/                                                                            \
{                                                                             \
  M##_size_t           array_size;                                            \
  M##_size_t           i;                                                     \
  void M               *arr_ptr;                                              \
                                                                              \
  an_eh_stack_entry             ehse;                                         \
  an_array_alloc_eh_info        aaehi;                                        \
  a_boolean                     create_eh_stack_entry;                        \
  a_boolean                     free_memory_on_cleanup = array_ptr == NULL;   \
                                                                              \
  /* An entry is created on the EH stack if the class for which the           \
     array is being created has a destructor, or if the object is             \
     dynamically allocated. */                                                \
  create_eh_stack_entry = dtor != NULL || array_ptr == NULL;                  \
  if (array_ptr == NULL || record_array_info) {                               \
    a_boolean   err;                                                          \
    array_size = number_of_elements * element_size;                           \
    if (array_ptr == NULL) {                                                  \
      /* Allocate the array if a pointer has not been supplied by the         \
         caller. */                                                           \
      array_ptr = M##_alloc_array(array_size, new_routine);                   \
      if (array_ptr == NULL) {                                                \
        goto error_exit;                                                      \
      }  /* if */                                                             \
    }  /* if */                                                               \
    /* Record the array size information so that the array can be properly    \
       freed later. */                                                        \
    err = M##_record_array_alloc_info(                                        \
                               array_ptr, array_size, number_of_elements);    \
    if (err) goto error_exit;                                                 \
  }  /* if */                                                                 \
  if (create_eh_stack_entry) {                                                \
    add_vec_new_or_delete_eh_stack_entry(&ehse, &aaehi, /*is_vec_new=*/TRUE); \
    aaehi.free_memory_on_cleanup = free_memory_on_cleanup;                    \
    aaehi.number_of_elements     = number_of_elements;                        \
    aaehi.element_size           = element_size;                              \
    aaehi.destructor             = dtor;                                      \
    aaehi.delete_routine         = delete_routine;                            \
    aaehi.is_two_arg             = is_two_arg;                                \
    aaehi.array_ptr              = array_ptr;                                 \
  }  /* if */                                                                 \
  /* Call the constructor, if any, for each member of the array.  Note that   \
     there may be zero elements.  Cfront tacks on what appears to be eight    \
     additional NULL values to be used as the addresses of the first          \
     eight virtual base classes.  The EDG compiler generates a special wrapper\
     for use by vec_new and doesn't need the additional arguments.  The       \
     additional arguments here allow cfront-generated vec_new calls to        \
     be used with this vec_new. */                                            \
  if (ctor != NULL) {                                                         \
    for (i = 0, arr_ptr = array_ptr;                                          \
         i < number_of_elements;                                              \
         i++, increment_ptr(M, arr_ptr, element_size)) {                      \
     if (src_array_ptr == NULL) {                                             \
       /* Call the default constructor. */                                    \
        (*ctor)(arr_ptr);                                                     \
      } else {                                                                \
        /* Call the copy constructor. */                                      \
        M##_copy_constructor_ptr  cctor;                                      \
        cctor = (M##_copy_constructor_ptr)ctor;                               \
        (*cctor)(arr_ptr, src_array_ptr);                                     \
      }  /* if */                                                             \
      if (dtor != NULL) {                                                     \
        /* Update the counter of the number of elements processed in the      \
           EH stack entry. */                                                 \
        aaehi.elements_processed++;                                           \
      }  /* if */                                                             \
      /* Go to the next element in the source array when the constructor      \
         being called is a copy constructor. */                               \
      if (src_array_ptr != NULL) {                                            \
        increment_ptr(M, src_array_ptr, element_size);                        \
      } /* if */                                                              \
    }  /* for */                                                              \
  }  /* if */                                                                 \
  if (create_eh_stack_entry) {                                                \
    /* Unlink the vec_new EH stack entry. */                                  \
    __curr_eh_stack_entry = __curr_eh_stack_entry->next;                      \
  }  /* if */                                                                 \
error_exit:                                                                   \
  /* Return the pointer to the array. */                                      \
  return array_ptr;                                                           \
}  /* array_new_general */
__VAR_MEMORY_LIST1__()
#undef __VAR_MEM_HELPER1__

#else /* !EXCEPTION_HANDLING */

#pragma diag_suppress = Pe826

/*ARGSUSED*/ /* <-- "dtor" is only used when EXCEPTION_HANDLING is TRUE. */
#define __VAR_MEM_HELPER1__(M, I)                                             \
static void M *M##_array_new_general(void M *            array_ptr,           \
                                     M##_size_t          number_of_elements,  \
                                     M##_size_t          element_size,        \
                                     void M *            src_array_ptr,       \
                                     M##_constructor_ptr ctor,                \
                                     M##_destructor_ptr  dtor,                \
                                     M##_new_ptr         new_routine,         \
                                     M##_delete_ptr      delete_routine,      \
                                     int                 is_two_arg,          \
                                     a_boolean           record_array_info)   \
/*                                                                            \
Allocate storage for an array, then call a constructor for each               \
element of the array.  If array_ptr is NULL, allocate the space for an        \
array of class objects (with number_of_elements elements each of size         \
element_size).  Also remember the size of the array in a                      \
behind-the-scenes data structure so that it can be recalled at the            \
time of the corresponding vec_delete call.  If array_ptr is non-NULL,         \
it points to an already-allocated array.  If ctor is non-NULL, it             \
points to a constructor function to be called for each element of the         \
array (whether the array is allocated here or pre-allocated).  Return         \
the address of the array.                                                     \
                                                                              \
src_array_ptr points to the array to be copied when ctor points to a          \
copy constructor.                                                             \
                                                                              \
dtor is a pointer to the destructor for objects of the element type.          \
This is used by the exception handling mechanism for object cleanup           \
if an exception is thrown while the array is being constructed.               \
If there is no destructor then dtor is NULL and no cleanup is done.           \
                                                                              \
delete_routine is a pointer to the delete routine to be used to deallocate    \
the space in the event that an exception is thrown during construction.       \
is_two_arg is TRUE if delete_routine refers to a two argument version         \
of the delete operator.  record_array_info is TRUE if the array size          \
information should be saved even though an array_ptr value was provided.      \
                                                                              \
This routine needs to record the size of the array that was allocated so      \
that the size is known when the array is deallocated.  One of two means       \
of recording this information is used depending on the setting of the         \
configuration flag USE_PREFIX_FOR_ARRAY_ALLOC_INFO.  If this flag is TRUE,    \
a prefix structure is allocated as part of the array allocation, and this     \
prefix is used to store the size.  The pointer returned to the caller points  \
to the memory after the prefix block.                                         \
                                                                              \
When USE_PREFIX_FOR_ARRAY_ALLOC_INFO is FALSE, a linked list is used          \
to record the number of elements in the array.  Consequently, the             \
performance degrades if a large number of arrays are allocated.  For          \
this reason, the alternate mechanism is recommended for production            \
use.                                                                          \
*/                                                                            \
{                                                                             \
  M##_size_t           array_size;                                            \
  M##_size_t           i;                                                     \
  void M               *arr_ptr;                                              \
                                                                              \
  if (array_ptr == NULL || record_array_info) {                               \
    a_boolean   err;                                                          \
    array_size = number_of_elements * element_size;                           \
    if (array_ptr == NULL) {                                                  \
      /* Allocate the array if a pointer has not been supplied by the         \
         caller. */                                                           \
      array_ptr = M##_alloc_array(array_size, new_routine);                   \
      if (array_ptr == NULL) {                                                \
        goto error_exit;                                                      \
      }  /* if */                                                             \
    }  /* if */                                                               \
    /* Record the array size information so that the array can be properly    \
       freed later. */                                                        \
    err = M##_record_array_alloc_info(                                        \
                               array_ptr, array_size, number_of_elements);    \
    if (err) goto error_exit;                                                 \
  }  /* if */                                                                 \
  /* Call the constructor, if any, for each member of the array.  Note that   \
     there may be zero elements.  Cfront tacks on what appears to be eight    \
     additional NULL values to be used as the addresses of the first          \
     eight virtual base classes.  The EDG compiler generates a special wrapper\
     for use by vec_new and doesn't need the additional arguments.  The       \
     additional arguments here allow cfront-generated vec_new calls to        \
     be used with this vec_new. */                                            \
  if (ctor != NULL) {                                                         \
    for (i = 0, arr_ptr = array_ptr;                                          \
         i < number_of_elements;                                              \
         i++, increment_ptr(M, arr_ptr, element_size)) {                      \
     if (src_array_ptr == NULL) {                                             \
       /* Call the default constructor. */                                    \
        (*ctor)(arr_ptr);                                                     \
      } else {                                                                \
        /* Call the copy constructor. */                                      \
        M##_copy_constructor_ptr  cctor;                                      \
        cctor = (M##_copy_constructor_ptr)ctor;                               \
        (*cctor)(arr_ptr, src_array_ptr);                                     \
      }  /* if */                                                             \
      /* Go to the next element in the source array when the constructor      \
         being called is a copy constructor. */                               \
      if (src_array_ptr != NULL) {                                            \
        increment_ptr(M, src_array_ptr, element_size);                        \
      } /* if */                                                              \
    }  /* for */                                                              \
  }  /* if */                                                                 \
error_exit:                                                                   \
  /* Return the pointer to the array. */                                      \
  return array_ptr;                                                           \
}  /* array_new_general */
__VAR_MEMORY_LIST1__()
#undef __VAR_MEM_HELPER1__

#pragma diag_default = Pe826

#endif /* EXCEPTION_HANDLING */

#if ABI_CHANGES_FOR_ARRAY_NEW_AND_DELETE
#define __HEAP_MEM_HELPER1__(M, I)                                            \
EXTERN_C void M *M##_array_new(M##_size_t          number_of_elements,        \
                               M##_size_t          element_size,              \
                               M##_constructor_ptr ctor,                      \
                               M##_destructor_ptr  dtor,                      \
                               M##_new_ptr         new_routine,               \
                               M##_delete_ptr      delete_routine,            \
                               int                 is_two_arg)                \
/*                                                                            \
This entry point is used for operations that use class specific array         \
new and delete operators.  The new and delete routines are pointed to         \
by new_routine and delete_routine.  is_two_arg is TRUE if the delete          \
routine is one that requires two arguments.                                   \
*/                                                                            \
{                                                                             \
  if (new_routine == NULL)                                                    \
    new_routine = default_array_operator_new(M);                              \
  if (delete_routine == NULL)                                                 \
    delete_routine = default_array_operator_delete;                           \
  return M##_array_new_general(NULL, number_of_elements, element_size,        \
                               NULL, ctor, dtor, new_routine,                 \
                               delete_routine, is_two_arg,                    \
                               /*record_array_info=*/FALSE);                  \
}  /* __array_new */
__HEAP_MEMORY_LIST1__()
#undef __HEAP_MEM_HELPER1__
#endif /* ABI_CHANGES_FOR_ARRAY_NEW_AND_DELETE */


#if ABI_COMPATIBILITY_VERSION >= 234
#define __VAR_MEM_HELPER1__(M, I)                                             \
EXTERN_C void M *M##_placement_array_new(                                     \
                           void M *            array_ptr,                     \
                           M##_size_t          number_of_elements,            \
                           M##_size_t          element_size,                  \
                           M##_constructor_ptr ctor,                          \
                           M##_destructor_ptr  dtor)                          \
/*                                                                            \
This entry point is used for placement array new operations.  The actual      \
memory is allocated by a call to the appropriate new routine before           \
this routine is called.  This routine is used to record the array size        \
information and to call the constructor for each array element.               \
*/                                                                            \
{                                                                             \
  return M##_array_new_general(array_ptr, number_of_elements, element_size,   \
                               NULL, ctor, dtor, (M##_new_ptr)NULL,           \
                               (M##_delete_ptr)NULL, /*is_two_arg=*/FALSE,    \
                               /*record_array_info=*/TRUE);                   \
}  /* __placement_array_new */
__VAR_MEMORY_LIST1__()
#undef __VAR_MEM_HELPER1__
#endif /* ABI_COMPATIBILITY_VERSION >= 234 */



#if EXCEPTION_HANDLING
EXTERN_C void *__vec_new_eh(void                         *array_ptr,
                            __vec_element_count_t        number_of_elements,
                            size_t                       element_size,
                            a_constructor_ptr            ctor,
                            a_destructor_ptr             dtor)
/*
This entry point is used by code that uses exception handling for
new operations that do not involve the use of a class specific
operator new.
*/
{
  return (array_new_general(array_ptr, number_of_elements, element_size,
                            (void*)NULL, ctor, dtor,
                            default_array_operator_new,
                            default_array_operator_delete,
                            /*is_two_arg=*/FALSE,
                            /*record_array_info=*/FALSE));
}  /* __vec_new_eh */
#endif /* EXCEPTION_HANDLING */


#define __HEAP_MEM_HELPER1__(M, I)                                            \
EXTERN_C void M *M##_vec_new(void M *            array_ptr,                   \
                             M##_size_t          number_of_elements,          \
                             M##_size_t          element_size,                \
                             M##_constructor_ptr ctor)                        \
/*                                                                            \
This is an entry point used for compatibility with code generated             \
before EH was supported.  This is similar to vec_new_eh, except that          \
no destructor pointer is provided.                                            \
*/                                                                            \
{                                                                             \
  return M##_array_new_general(array_ptr, number_of_elements, element_size,   \
                               NULL, ctor, /*a_destructor_ptr*/NULL,          \
                               default_array_operator_new(M),                 \
                               default_array_operator_delete,                 \
                               /*is_two_arg=*/FALSE,                          \
                               /*record_array_info=*/FALSE);                  \
}  /* __vec_new */
__HEAP_MEMORY_LIST1__()
#undef __HEAP_MEM_HELPER1__


#define __VAR_MEM_HELPER1__(M, I)                                             \
EXTERN_C void M *M##_vec_construct(void M *        array_ptr,                 \
                               M##_size_t          number_of_elements,        \
                               M##_size_t          element_size,              \
                               M##_constructor_ptr ctor)                      \
/*                                                                            \
This is for the case of non-heap vector construction.                         \
*/                                                                            \
{                                                                             \
  return M##_array_new_general(array_ptr, number_of_elements, element_size,   \
                               NULL, ctor, /*a_destructor_ptr*/NULL,          \
                               (M##_new_ptr)NULL, (M##_delete_ptr)NULL,       \
                               /*is_two_arg=*/FALSE,                          \
                               /*record_array_info=*/FALSE);                  \
}  /* __vec_construct */
__VAR_MEMORY_LIST1__()
#undef __VAR_MEM_HELPER1__


#if EXCEPTION_HANDLING
EXTERN_C void __vec_cctor_eh(void                       *array_ptr,
                             __vec_element_count_t      number_of_elements,
                             size_t                     element_size,
                             a_copy_constructor_ptr     ctor,
                             void                       *src_array_ptr,
                             a_destructor_ptr           dtor)
/*
This is an entry point to array_new_general used to call the copy
constructor for each element of an array.  The corresponding element of
the array pointed to by src_array_ptr is the source operand for the
copy constructor.  Because this runtime routine will only be called for
constructor initialization of member arrays, the number_of_elements
can never be zero.
*/
{
  (void)array_new_general(array_ptr, number_of_elements, element_size,
                          src_array_ptr, (a_constructor_ptr)ctor, dtor,
                          (a_new_ptr)NULL, (a_delete_ptr)NULL,
                          /*is_two_arg=*/FALSE,
                          /*record_array_info=*/FALSE);
}  /* __vec_ctor_eh */


EXTERN_C void __cleanup_vec_new_or_delete(an_eh_stack_entry_ptr ehsep)
/*
Called by the exception handling cleanup routine to do the cleanup
processing for a vec_new or vec_delete operation that was interrupted by
an exception.
*/
{
  /* Call the destructor, if specified, on each element in the array, in
     reverse order. */
  an_array_alloc_eh_info_ptr    aaehip = ehsep->variant.array_alloc_eh_info;
  a_destructor_ptr              dtor = aaehip->destructor;
  a_sizeof_t                    number_of_elements;
  a_sizeof_t                    element_size;
  void*                         arr_ptr;
  void*                         array_ptr;
  a_sizeof_t                    i;
  a_sizeof_t                    first_element;

  array_ptr = (void *)aaehip->array_ptr;
  element_size = aaehip->element_size;
  if (aaehip->is_vec_new) {
    /* Cleaning up a vec_new.  Destroy the fully constructed elements of
       the array in reverse order. */
    number_of_elements = aaehip->elements_processed;
    first_element = number_of_elements - 1;
  } else {
    first_element = aaehip->number_of_elements -
                    aaehip->elements_processed - 1;
    number_of_elements = first_element + 1;
  }  /* if */
  if (dtor != NULL) {
    /* If there is a destructor, destroy the objects. */
    for (i = 0,
         arr_ptr = (void *)(((char *)array_ptr) +
                                                first_element * element_size);
         i < number_of_elements;
         i++, increment_ptr(arr_ptr, -(int)(element_size))) {
      /* We always have a destructor that needs no extra argument here. */
      (*dtor)(arr_ptr);
    }  /* for */
  }  /* if */
  if (aaehip->free_memory_on_cleanup) {
    /* Call the routine to free the memory. */
    size_t      size = element_size * number_of_elements;
    free_array(array_ptr, size, aaehip->delete_routine, aaehip->is_two_arg);
  }  /* if */
}  /* __cleanup_vec_new_or_delete */
#endif /* EXCEPTION_HANDLING */


#if EXCEPTION_HANDLING

#define __VAR_MEM_HELPER1__(M, I)                                             \
static void M##_array_delete_general(void M *           array_ptr,            \
                                     M##_size_t         number_of_elements,   \
                                     M##_size_t         element_size,         \
                                     M##_destructor_ptr dtor,                 \
                                     M##_delete_ptr     delete_routine,       \
                                     int                is_two_arg)           \
                                                                              \
/*                                                                            \
Call a destructor for each element of an array, then delete the storage       \
for the array.  array_ptr points to the array, which has number_of_elements   \
elements each of size element_size.  If number_of_elements is -1, use the     \
size stored by vec_new at the time of allocation of this array.               \
If array_ptr is NULL, this routine does nothing and returns.                  \
If dtor is non-NULL, it points to a destructor function to be called          \
for each element of the array.  If delete_flag is TRUE, the storage           \
for the array is deallocated after the destruction; number_of_elements        \
must be -1 for that case.                                                     \
*/                                                                            \
{                                                                             \
  M##_size_t i;                                                               \
  void M *   arr_ptr;                                                         \
  M##_size_t array_size = 0;                                                  \
                                                                              \
  /* If the address of the array is NULL, do nothing. */                      \
  if (array_ptr != NULL ) {                                                   \
    an_eh_stack_entry           ehse;                                         \
    an_array_alloc_eh_info      aaehi;                                        \
    add_vec_new_or_delete_eh_stack_entry(&ehse, &aaehi, /*is_vec_new=*/FALSE);\
    aaehi.free_memory_on_cleanup = delete_routine != NULL;                    \
    aaehi.array_ptr              = array_ptr;                                 \
    aaehi.number_of_elements     = number_of_elements;                        \
    aaehi.element_size           = element_size;                              \
    aaehi.destructor             = dtor;                                      \
    aaehi.delete_routine         = delete_routine;                            \
    aaehi.is_two_arg             = is_two_arg;                                \
    /* Determine the number of elements in the array, if unknown. */          \
    if (number_of_elements == (M##_size_t) -1) {                              \
      /* Determine the number of elements from the memory allocation size. */ \
      array_size = M##_get_array_size(array_ptr, element_size);               \
      number_of_elements = array_size / element_size;                         \
    }  /* if */                                                               \
    aaehi.number_of_elements     = number_of_elements;                        \
                                                                              \
    /* Call the destructor, if specified, on each element in the array, in    \
       reverse order. */                                                      \
    if (dtor != NULL) {                                                       \
      for (i = 0,                                                             \
           arr_ptr = ((char M *)array_ptr +                                   \
                      (number_of_elements - 1) * element_size);               \
           i < number_of_elements;                                            \
           i++, increment_ptr(M, arr_ptr, -(int)(element_size))) {            \
        /* Update the counter of the number of elements processed in the      \
           EH stack entry.  This is incremented before the destructor is      \
           called so that, should an exception occur, we won't try            \
           destroying this element again. */                                  \
        aaehi.elements_processed++;                                           \
        /* We always have a destructor that needs no extra argument here. */  \
        (*dtor)(arr_ptr);                                                     \
      }  /* for */                                                            \
    }  /* if */                                                               \
    /* Unlink the vec_new EH stack entry.  This is unlinked before the memory \
       for the array is freed.  If an exception occurs during the free        \
       it should just be handled by the normal mechanism. */                  \
    __curr_eh_stack_entry = __curr_eh_stack_entry->next;                      \
    /* Delete the array, if requested. */                                     \
    if (delete_routine) {                                                     \
      M##_free_array(array_ptr, array_size, delete_routine, is_two_arg);      \
    }  /* if */                                                               \
  }  /* if */                                                                 \
}  /* array_delete_general */
__VAR_MEMORY_LIST1__()
#undef __VAR_MEM_HELPER1__

#else /* !EXCEPTION_HANDLING */

#define __VAR_MEM_HELPER1__(M, I)                                             \
static void M##_array_delete_general(void M *           array_ptr,            \
                                     M##_size_t         number_of_elements,   \
                                     M##_size_t         element_size,         \
                                     M##_destructor_ptr dtor,                 \
                                     M##_delete_ptr     delete_routine,       \
                                     int                is_two_arg)           \
                                                                              \
/*                                                                            \
Call a destructor for each element of an array, then delete the storage       \
for the array.  array_ptr points to the array, which has number_of_elements   \
elements each of size element_size.  If number_of_elements is -1, use the     \
size stored by vec_new at the time of allocation of this array.               \
If array_ptr is NULL, this routine does nothing and returns.                  \
If dtor is non-NULL, it points to a destructor function to be called          \
for each element of the array.  If delete_flag is TRUE, the storage           \
for the array is deallocated after the destruction; number_of_elements        \
must be -1 for that case.                                                     \
*/                                                                            \
{                                                                             \
  M##_size_t i;                                                               \
  void M *   arr_ptr;                                                         \
  M##_size_t array_size = 0;                                                  \
                                                                              \
  /* If the address of the array is NULL, do nothing. */                      \
  if (array_ptr != NULL ) {                                                   \
    /* Determine the number of elements in the array, if unknown. */          \
    if (number_of_elements == (M##_size_t) -1) {                              \
      /* Determine the number of elements from the memory allocation size. */ \
      array_size = M##_get_array_size(array_ptr, element_size);               \
      number_of_elements = array_size / element_size;                         \
    }  /* if */                                                               \
                                                                              \
    /* Call the destructor, if specified, on each element in the array, in    \
       reverse order. */                                                      \
    if (dtor != NULL) {                                                       \
      for (i = 0,                                                             \
           arr_ptr = ((char M *)array_ptr +                                   \
                      (number_of_elements - 1) * element_size);               \
           i < number_of_elements;                                            \
           i++, increment_ptr(M, arr_ptr, -(int)(element_size))) {            \
        /* We always have a destructor that needs no extra argument here. */  \
        (*dtor)(arr_ptr);                                                     \
      }  /* for */                                                            \
    }  /* if */                                                               \
    /* Delete the array, if requested. */                                     \
    if (delete_routine) {                                                     \
      M##_free_array(array_ptr, array_size, delete_routine, is_two_arg);      \
    }  /* if */                                                               \
  }  /* if */                                                                 \
}  /* array_delete_general */
__VAR_MEMORY_LIST1__()
#undef __VAR_MEM_HELPER1__

#endif /* EXCEPTION_HANDLING */

#pragma diag_suppress = Pe826
/*ARGSUSED*/ /* <-- "unused" is unused. */
#define __HEAP_MEM_HELPER1__(M, I)                                            \
EXTERN_C void M##_vec_delete(void M *           array_ptr,                    \
                             M##_size_t         number_of_elements,           \
                             M##_size_t         element_size,                 \
                             M##_destructor_ptr dtor,                         \
                             int                delete_flag,                  \
                             int                unused)                       \
/*                                                                            \
Entry point used for the normal vector delete operation.  The unused          \
parameter is there for cfront compatibility.                                  \
*/                                                                            \
{                                                                             \
  M##_delete_ptr         delete_routine = NULL;                               \
  if (delete_flag)                                                            \
    delete_routine = default_array_operator_delete;                           \
  M##_array_delete_general(array_ptr, number_of_elements, element_size, dtor, \
                           delete_routine,                                    \
                           /*is_two_arg=*/FALSE);                             \
}  /* __vec_delete */
__HEAP_MEMORY_LIST1__()
#undef __HEAP_MEM_HELPER1__
#pragma diag_default = Pe826

// This is for the case of a non-heap vector
#define __VAR_MEM_HELPER1__(M, I)                                             \
EXTERN_C void M##_vec_destroy(void M                  *array_ptr,             \
                              M##_size_t              number_of_elements,     \
                              M##_size_t              element_size,           \
                              M##_destructor_ptr      dtor)                   \
/*                                                                            \
Entry point used for the normal vector delete operation.  The unused          \
parameter is there for cfront compatibility.                                  \
*/                                                                            \
{                                                                             \
  M##_array_delete_general(array_ptr, number_of_elements, element_size, dtor, \
                           (M##_delete_ptr)NULL,                              \
                           /*is_two_arg=*/FALSE);                             \
}  /* __vec_destroy */
__VAR_MEMORY_LIST1__()
#undef __VAR_MEM_HELPER1__


#if ABI_CHANGES_FOR_ARRAY_NEW_AND_DELETE
#define __VAR_MEM_HELPER1__(M, I)                                             \
EXTERN_C void M##_array_delete(void M                  *array_ptr,            \
                               M##_size_t              number_of_elements,    \
                               M##_size_t              element_size,          \
                               M##_destructor_ptr      dtor,                  \
                               M##_delete_ptr          delete_routine,        \
                               int                     is_two_arg)            \
/*                                                                            \
This entry point is used for operations that use class specific array         \
new and delete operators.  The delete routine is pointed to                   \
by delete_routine.  is_two_arg is TRUE if the delete routine is one that      \
requires two arguments.                                                       \
*/                                                                            \
{                                                                             \
  M##_array_delete_general(array_ptr, number_of_elements, element_size, dtor, \
                           delete_routine, is_two_arg);                       \
}  /* __array_delete */
__VAR_MEMORY_LIST1__()
#undef __VAR_MEM_HELPER1__
#endif /* ABI_CHANGES_FOR_ARRAY_NEW_AND_DELETE */

                                     
EXTERN_C void _array_pointer_not_from_vec_new()
/*
This routine is used when a pointer that was not created by vec_new is
passed to one other vector handling routines that needs to get the size
from the information created by vec_new.  This routine simply aborts.
The name is intended to describe the nature of the problem to the user
*/
{
        abort();
}

/******************************************************************************
*                                                             \  ___  /       *
*                                                               /   \         *
* Edison Design Group C++ Runtime                            - | \^/ | -      *
*                                                               \   /         *
* Proprietary information of Edison Design Group Inc.         /  | |  \       *
* Copyright 1992-1996 Edison Design Group Inc.                   [_]          *
*                                                                             *
******************************************************************************/
