/* __write_buffered function. */
/* Copyright (C) 2003 IAR Systems.  All rights reserved. */

/* This is a buffered write mechanism for debug builds only. */

#include "yfuns.h"

_STD_BEGIN

#define STORE_SIZE 80

static size_t storeLen = 0;
static unsigned char store[STORE_SIZE];

/* Returns 0 or _LLIO_ERROR. */
static size_t flush(void)
{
  if (storeLen > 0)
  {
    size_t s = storeLen;
    storeLen = 0;

    if (__dwrite(_LLIO_STDOUT, store, s) == _LLIO_ERROR)
    {
      return _LLIO_ERROR;
    }
  }
  return 0;
}

#pragma required = __cstart_closeall
size_t __write_buffered(int handle, const unsigned char * buf, size_t size)
{
  /* Flush our buffer. */
  if (buf == 0)
  {
    if (   handle == _LLIO_STDOUT
        || handle == -1)
    {
      return flush();
    }
    return 0;
  }

  /* Only standard out is buffered. */
  if (handle != _LLIO_STDOUT)
  {
    return __dwrite(handle, buf, size);
  }

  /*
   * In case the buf is larger than our store then the most efficient
   * we can do is to pass the entire buffer to __dwrite.  (We need to
   * flush our store first.)
   */

  if (size > STORE_SIZE)
  {
    if (flush() == _LLIO_ERROR)
    {
      return _LLIO_ERROR;
    }
    if (__dwrite(handle, buf, size) == _LLIO_ERROR)
    {
      return _LLIO_ERROR;
    }

    return size;
  }
  else
  {
    /*
     * Fill up a buffer, if there are more characters to be written,
     * or if the buffer contains newline characters, then flush the
     * buffer.
     */

    size_t charsLeft = size;

    while (charsLeft)
    {
      int foundNewline = 0;
      /* Fill buffer. */
      while (charsLeft && storeLen < STORE_SIZE)
      {
        char ch = *buf++;
        if (ch == '\n')
        {
          foundNewline = 1;
        }
        store[storeLen++] = ch;
        --charsLeft;
      }

      if (charsLeft || foundNewline)
      {
        if (flush() == _LLIO_ERROR)
        {
          return _LLIO_ERROR;
        }
      }
    }
    return size;
  }
}

_STD_END
